"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const ejs_1 = require("ejs");
const getIncludeEjsDependencies = (context, content, options, nestedDependencyRoot) => {
    const dependencyPattern = /<%[_\W]?\s*include\([^%]*\)\s*[_\W]?%>/g;
    let matches = dependencyPattern.exec(content);
    const dependencies = [];
    while (matches) {
        // get filename with "" or ''
        const matchFilename = matches[0].match(/(['"`])[^'"`]*\1/);
        // get real file name without "" or ''
        let filename = matchFilename !== null ? matchFilename[0].replace(/['"`]/g, '') : null; // if filename is not null, then replace ""
        // if filename is not empty
        if (filename !== null) {
            // add .ejs if no ext
            let filePath = '';
            if (!filename.endsWith('.ejs')) {
                filename += '.ejs';
            }
            // get filepath
            if (nestedDependencyRoot) {
                filePath = /^\//.test(filename)
                    ? (0, path_1.resolve)(options.root ?? '', filename.replace(/^\//, ''))
                    : (0, path_1.resolve)(nestedDependencyRoot, filename);
            }
            else {
                filePath = /^\//.test(filename)
                    ? (0, path_1.resolve)(options.root ?? '', filename.replace(/^\//, ''))
                    : (0, path_1.resolve)(context.context, filename);
            }
            dependencies.push(filePath);
            // then check if there are any nested dependencies inside that dependendency;
            const fileContent = (0, fs_1.readFileSync)(filePath, 'utf8');
            const dependencyFolderPath = (0, path_1.dirname)(filePath);
            // recursive
            const nestDependencies = getIncludeEjsDependencies(context, fileContent, options, dependencyFolderPath);
            if (nestDependencies.length > 0) {
                dependencies.push(...nestDependencies);
            }
        }
        // check the content if there are no other dep.
        matches = dependencyPattern.exec(content);
    }
    return dependencies;
};
const requireFunction = (requestPath) => {
    if (requestPath.endsWith('.json')) {
        return JSON.parse((0, fs_1.readFileSync)(requestPath, 'utf-8'));
    }
    else {
        return require(requestPath);
    }
};
const resolveRequirePaths = async (context, content) => {
    const requirePattern = /<%[_\W]?.*(require\(.*\)).*[_\W]?%>/g;
    let resultContent = content;
    let matches = requirePattern.exec(content);
    while (matches) {
        const matchFilename = matches[1].match(/(['"`])[^'"`]*\1/);
        const requestSource = matchFilename !== null ? matchFilename[0].replace(/['"`]/g, '') : null;
        if (requestSource !== null) {
            const result = await context.getResolve()(context.context, requestSource);
            resultContent = resultContent.replace(matches[1], `require('${result}')`);
        }
        matches = requirePattern.exec(content);
    }
    return resultContent;
};
async function ejsLoader(content, sourceMap, additionalData) {
    const callback = this.async();
    const loaderOptions = this.getOptions();
    if (loaderOptions.data !== undefined && typeof loaderOptions.data === 'string') {
        Object.assign(loaderOptions, { data: JSON.parse(loaderOptions.data) });
    }
    const ejsOptions = Object.assign({
        filename: this.resourcePath,
    }, loaderOptions);
    if (Object.keys(loaderOptions.data ?? {}).includes('require')) {
        callback(Error('Do not set "require" in the loader options'));
    }
    const currentHtmlWebpackPlugin = this._compiler?.options.plugins.filter((plugin) => typeof plugin === 'object' &&
        plugin.options &&
        plugin.options.template &&
        plugin.options.template === this.resource)[0];
    const templateParameters = {};
    if (typeof currentHtmlWebpackPlugin === 'object' &&
        typeof currentHtmlWebpackPlugin.options.templateParameters !== 'function') {
        Object.assign(templateParameters, {
            htmlWebpackPlugin: {
                options: currentHtmlWebpackPlugin.options.templateParameters,
            },
        });
    }
    const parameter = Object.assign({
        require: (source) => requireFunction(source),
    }, loaderOptions.data, templateParameters);
    try {
        const resolveContent = await resolveRequirePaths(this, content);
        const template = await (0, ejs_1.compile)(resolveContent, ejsOptions)(parameter);
        const ejsDependencies = getIncludeEjsDependencies(this, content, ejsOptions);
        ejsDependencies.forEach((dependency) => {
            this.addDependency(dependency);
        });
        callback(null, template, sourceMap, additionalData);
    }
    catch (error) {
        callback(error);
    }
}
exports.default = ejsLoader;
//# sourceMappingURL=index.js.map